import { Meta, StoryObj, componentWrapperDecorator, moduleMetadata } from "@storybook/angular";

import { SectionComponent } from "../section";
import { TypographyModule } from "../typography";

import { CardComponent } from "./card.component";

export default {
  title: "Component Library/Card",
  component: CardComponent,
  decorators: [
    moduleMetadata({
      imports: [TypographyModule, SectionComponent],
    }),
    componentWrapperDecorator(
      (story) => `<div class="tw-bg-background-alt tw-p-10 tw-text-main">${story}</div>`,
    ),
  ],
} as Meta;

type Story = StoryObj<CardComponent>;

/** Cards are presentational containers. */
export const Default: Story = {
  render: (args) => ({
    props: args,
    template: /*html*/ `
        <bit-card>
            <p bitTypography="body1" class="!tw-mb-0">Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras vitae congue risus. Interdum et malesuada fames ac ante ipsum primis in faucibus. Nunc elementum odio nibh, eget pellentesque sem ornare vitae. Etiam vel ante et velit fringilla egestas a sed sem. Fusce molestie nisl et nisi accumsan dapibus. Interdum et malesuada fames ac ante ipsum primis in faucibus. Sed eu risus ex. </p>
        </bit-card>
    `,
  }),
};

/** Cards are often paired with [Sections](/docs/component-library-section--docs). */
export const WithinSections: Story = {
  render: (args) => ({
    props: args,
    template: /*html*/ `
          <bit-section>
            <h2 bitTypography="h5">Bar</h2>
            <bit-card>
                <p bitTypography="body1" class="!tw-mb-0">Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras vitae congue risus. Interdum et malesuada fames ac ante ipsum primis in faucibus. Nunc elementum odio nibh, eget pellentesque sem ornare vitae. Etiam vel ante et velit fringilla egestas a sed sem. Fusce molestie nisl et nisi accumsan dapibus. Interdum et malesuada fames ac ante ipsum primis in faucibus. Sed eu risus ex. </p>
            </bit-card>
          </bit-section>

          <bit-section>
            <h2 bitTypography="h5">Bar</h2>
            <bit-card>
                <p bitTypography="body1" class="!tw-mb-0">Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras vitae congue risus. Interdum et malesuada fames ac ante ipsum primis in faucibus. Nunc elementum odio nibh, eget pellentesque sem ornare vitae. Etiam vel ante et velit fringilla egestas a sed sem. Fusce molestie nisl et nisi accumsan dapibus. Interdum et malesuada fames ac ante ipsum primis in faucibus. Sed eu risus ex. </p>
            </bit-card>
          </bit-section>

          <bit-section>
            <h2 bitTypography="h5">Bar</h2>
            <bit-card>
                <p bitTypography="body1" class="!tw-mb-0">Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras vitae congue risus. Interdum et malesuada fames ac ante ipsum primis in faucibus. Nunc elementum odio nibh, eget pellentesque sem ornare vitae. Etiam vel ante et velit fringilla egestas a sed sem. Fusce molestie nisl et nisi accumsan dapibus. Interdum et malesuada fames ac ante ipsum primis in faucibus. Sed eu risus ex. </p>
            </bit-card>
          </bit-section>
      `,
  }),
};
